@@ -22,19 +22,22 @@
 
 #![allow(dead_code)]
 
+use std::cell::RefCell;
 use std::collections::HashMap;
 
+use super::cellvalue::CellValue;
 use super::conn_params::ConnParams;
 use super::error::Error;
-use super::param::Param;
+use super::params::Param;
 use super::statement::Statement;
 use super::transaction::*;
 use super::wireprotocol::*;
+use super::xsqlvar::XSQLVar;
 use super::*;
 
 pub struct Connection {
-    pub(crate) wp: WireProtocol,
-    pub(crate) trans_handle: i32, // transaction for operating from connection methods
+    wp: RefCell<WireProtocol>,
+    trans_handle: i32, // transaction for operating from connection methods
     conn_params: ConnParams,
     conn_options: HashMap<String, String>,
 }
@@ -72,7 +75,7 @@ impl Connection {
         let (trans_handle, _, _) = wp.op_response()?;
 
         Ok(Connection {
-            wp,
+            wp: RefCell::new(wp),
             trans_handle,
             conn_params,
             conn_options,
@@ -114,7 +117,7 @@ impl Connection {
         let (trans_handle, _, _) = wp.op_response()?;
 
         Ok(Connection {
-            wp,
+            wp: RefCell::new(wp),
             trans_handle,
             conn_params,
             conn_options,
@@ -122,11 +125,13 @@ impl Connection {
     }
 
     pub(crate) fn _execute_batch(&mut self, query: &str, trans_handle: i32) -> Result<(), Error> {
-        self.wp.op_exec_immediate(trans_handle, query)?;
-        self.wp.op_response()?;
+        let mut wp = self.wp.borrow_mut();
+        wp.op_exec_immediate(trans_handle, query)?;
+        wp.op_response()?;
 
         // commit automatically
-        self.commit()?;
+        wp.op_commit_retaining(trans_handle)?;
+        wp.op_response()?;
 
         Ok(())
     }
@@ -141,27 +146,30 @@ impl Connection {
         params: Vec<Param>,
         trans_handle: i32,
     ) -> Result<(), Error> {
-        self.wp.op_allocate_statement()?;
+        let mut stmt = {
+            let mut wp = self.wp.borrow_mut();
+            wp.op_allocate_statement()?;
 
-        let mut stmt_handle = if self.wp.accept_type == PTYPE_LAZY_SEND {
-            self.wp.lazy_response_count += 1;
-            -1
-        } else {
-            let (stmt_handle, _, _) = self.wp.op_response()?;
-            stmt_handle
+            let mut stmt_handle = if wp.accept_type == PTYPE_LAZY_SEND {
+                wp.lazy_response_count += 1;
+                -1
+            } else {
+                let (stmt_handle, _, _) = wp.op_response()?;
+                stmt_handle
+            };
+
+            wp.op_prepare_statement(stmt_handle, trans_handle, query)?;
+            if wp.accept_type == PTYPE_LAZY_SEND && wp.lazy_response_count > 0 {
+                wp.lazy_response_count -= 1;
+                let (h, _, _) = wp.op_response()?;
+                stmt_handle = h;
+            }
+            let (_, buf, _) = wp.op_response()?;
+            let (stmt_type, xsqlda) = wp.parse_xsqlda(&buf, stmt_handle)?;
+
+            Statement::new(self, trans_handle, stmt_handle, stmt_type, xsqlda, true)
         };
 
-        self.wp
-            .op_prepare_statement(stmt_handle, trans_handle, query)?;
-        if self.wp.accept_type == PTYPE_LAZY_SEND && self.wp.lazy_response_count > 0 {
-            self.wp.lazy_response_count -= 1;
-            let (h, _, _) = self.wp.op_response()?;
-            stmt_handle = h;
-        }
-        let (_, buf, _) = self.wp.op_response()?;
-        let (stmt_type, xsqlda) = self.wp.parse_xsqlda(&buf, stmt_handle)?;
-        let mut stmt = Statement::new(self, trans_handle, stmt_handle, stmt_type, xsqlda, true);
-
         stmt.execute(params)?;
 
         Ok(())
@@ -171,19 +179,28 @@ impl Connection {
         self._execute(query, params, self.trans_handle)
     }
 
-    pub(crate) fn _commit(&mut self, trans_handle: i32) -> Result<(), Error> {
-        self.wp.op_commit_retaining(trans_handle)?;
-        self.wp.op_response()?;
+    pub(crate) fn _commit(&self, trans_handle: i32) -> Result<(), Error> {
+        let mut wp = self.wp.borrow_mut();
+        wp.op_commit_retaining(trans_handle)?;
+        wp.op_response()?;
         Ok(())
     }
 
-    pub fn commit(&mut self) -> Result<(), Error> {
+    pub fn commit(&self) -> Result<(), Error> {
         self._commit(self.trans_handle)
     }
 
+    pub(crate) fn _begin_trans(&mut self) -> Result<i32, Error> {
+        let mut wp = self.wp.borrow_mut();
+        wp.op_transaction(false)?;
+        let (trans_handle, _, _) = wp.op_response()?;
+        Ok(trans_handle)
+    }
+
     pub(crate) fn _rollback(&mut self, trans_handle: i32) -> Result<(), Error> {
-        self.wp.op_rollback_retaining(trans_handle)?;
-        self.wp.op_response()?;
+        let mut wp = self.wp.borrow_mut();
+        wp.op_rollback_retaining(trans_handle)?;
+        wp.op_response()?;
         Ok(())
     }
 
@@ -192,25 +209,25 @@ impl Connection {
     }
 
     pub fn _prepare(&mut self, query: &str, trans_handle: i32) -> Result<Statement, Error> {
-        self.wp.op_allocate_statement()?;
+        let mut wp = self.wp.borrow_mut();
+        wp.op_allocate_statement()?;
 
-        let mut stmt_handle = if self.wp.accept_type == PTYPE_LAZY_SEND {
-            self.wp.lazy_response_count += 1;
+        let mut stmt_handle = if wp.accept_type == PTYPE_LAZY_SEND {
+            wp.lazy_response_count += 1;
             -1
         } else {
-            let (stmt_handle, _, _) = self.wp.op_response()?;
+            let (stmt_handle, _, _) = wp.op_response()?;
             stmt_handle
         };
 
-        self.wp
-            .op_prepare_statement(stmt_handle, trans_handle, query)?;
-        if self.wp.accept_type == PTYPE_LAZY_SEND && self.wp.lazy_response_count > 0 {
-            self.wp.lazy_response_count -= 1;
-            let (h, _, _) = self.wp.op_response()?;
+        wp.op_prepare_statement(stmt_handle, trans_handle, query)?;
+        if wp.accept_type == PTYPE_LAZY_SEND && wp.lazy_response_count > 0 {
+            wp.lazy_response_count -= 1;
+            let (h, _, _) = wp.op_response()?;
             stmt_handle = h;
         }
-        let (_, _, buf) = self.wp.op_response()?;
-        let (stmt_type, xsqlda) = self.wp.parse_xsqlda(&buf, stmt_handle)?;
+        let (_, _, buf) = wp.op_response()?;
+        let (stmt_type, xsqlda) = wp.parse_xsqlda(&buf, stmt_handle)?;
 
         Ok(Statement::new(
             self,
@@ -229,4 +246,55 @@ impl Connection {
     pub fn transaction(&mut self) -> Result<Transaction, Error> {
         Transaction::new(self)
     }
+
+    // methods for Statement
+
+    pub(crate) fn _execute_query(
+        &self,
+        stmt_handle: i32,
+        trans_handle: i32,
+        params: &[Param],
+    ) -> Result<(), Error> {
+        let mut wp = self.wp.borrow_mut();
+        wp.op_execute(stmt_handle, trans_handle, params)?;
+        wp.op_response()?;
+        Ok(())
+    }
+
+    pub(crate) fn _fetch(
+        &self,
+        stmt_handle: i32,
+        blr: &Vec<u8>,
+        xsqlda: &[XSQLVar],
+    ) -> Result<(Vec<Vec<CellValue>>, bool), Error> {
+        let mut wp = self.wp.borrow_mut();
+        wp.op_fetch(stmt_handle, &blr)?;
+        wp.op_fetch_response(xsqlda)
+    }
+
+    pub(crate) fn _get_blob_segments(
+        &self,
+        blob_id: &Vec<u8>,
+        trans_handle: i32,
+    ) -> Result<Vec<u8>, Error> {
+        let mut wp = self.wp.borrow_mut();
+        wp.get_blob_segments(blob_id, trans_handle)
+    }
+
+    pub(crate) fn _free_statement(&self, stmt_handle: i32, drop_type: i32) -> () {
+        let mut wp = self.wp.borrow_mut();
+        wp.op_free_statement(stmt_handle, drop_type).unwrap();
+        if wp.accept_type == PTYPE_LAZY_SEND {
+            wp.lazy_response_count += 1;
+        } else {
+            wp.op_response().unwrap();
+        }
+    }
+
+    // methods for Transaction
+    pub(crate) fn drop_transaction(&self, trans_handle: i32) -> () {
+        let mut wp = self.wp.borrow_mut();
+        wp.op_rollback(trans_handle).unwrap();
+        wp.op_response().unwrap();
+    }
 }
