@@ -22,16 +22,16 @@
 
 #![allow(dead_code)]
 
-use std::collections::{HashMap, HashSet};
-use std::io::prelude::*;
-
+use chrono::{Datelike, Timelike};
 use hex;
 use num_bigint::BigInt;
+use std::collections::{HashMap, HashSet};
+use std::io::prelude::*;
 
 use super::cellvalue::CellValue;
 use super::conn_params::ConnParams;
 use super::error::{Error, FirebirdError};
-use super::param::Param;
+use super::params::Param;
 use super::wirechannel::WireChannel;
 use super::xsqlvar::XSQLVar;
 use super::*;
@@ -1173,7 +1173,7 @@ impl WireProtocol {
         &mut self,
         stmt_handle: i32,
         trans_handle: i32,
-        params: &Vec<Param>,
+        params: &[Param],
     ) -> Result<(), Error> {
         debug_print!("op_execute()");
         self.pack_u32(OP_EXECUTE);
@@ -1204,7 +1204,7 @@ impl WireProtocol {
         &mut self,
         stmt_handle: i32,
         trans_handle: i32,
-        params: &Vec<Param>,
+        params: &[Param],
         output_blr: &[u8],
     ) -> Result<(), Error> {
         debug_print!("op_execute2()");
@@ -1420,7 +1420,7 @@ impl WireProtocol {
         }
     }
 
-    pub fn op_sql_response(&mut self, xsqlda: &[XSQLVar]) -> Result<Vec<CellValue>, Error> {
+    fn op_sql_response(&mut self, xsqlda: &[XSQLVar]) -> Result<Vec<CellValue>, Error> {
         debug_print!("op_sql_response()");
         let xsqlda_len = xsqlda.len();
         let mut row: Vec<CellValue> = Vec::with_capacity(xsqlda_len);
@@ -1499,7 +1499,7 @@ impl WireProtocol {
         Ok(blob_id)
     }
 
-    pub fn params_to_blr(&mut self, params: &Vec<Param>) -> Result<(Vec<u8>, Vec<u8>), Error> {
+    fn params_to_blr(&mut self, params: &[Param]) -> Result<(Vec<u8>, Vec<u8>), Error> {
         // Convert parameter array to BLR and values format.
         let mut values_list: Vec<u8> = Vec::new();
         let mut blr_list: Vec<u8> = Vec::new();
@@ -1532,7 +1532,7 @@ impl WireProtocol {
             let v: Vec<u8> = Vec::new();
             match p {
                 Param::Null => {
-                    values_list.write(&[14, 0, 0])?;
+                    blr_list.write(&[14, 0, 0])?;
                 }
                 Param::Text(s) => {
                     let b = s.as_bytes();
@@ -1540,13 +1540,77 @@ impl WireProtocol {
                     values_list.write(&v)?;
                     blr_list.write(&blr)?;
                 }
+                Param::Short(n) => {
+                    values_list.write(&utils::bint32_to_bytes(*n as i32))?;
+                    blr_list.write(&[8, 0])?;
+                }
                 Param::Long(n) => {
                     values_list.write(&utils::bint32_to_bytes(*n))?;
                     blr_list.write(&[8, 0])?;
                 }
-                _ => {
+                Param::Int64(n) => {
+                    values_list.write(&utils::bint64_to_bytes(*n))?;
+                    blr_list.write(&[16, 0])?;
+                }
+                Param::Int128(n) => {
+                    values_list.write(&utils::bint128_to_bytes(*n))?;
+                    blr_list.write(&[26, 0])?;
+                }
+                Param::Time(t) => {
+                    values_list.write(&utils::convert_time(
+                        t.hour(),
+                        t.minute(),
+                        t.second(),
+                        t.nanosecond(),
+                    ))?;
+                    blr_list.write(&[13])?;
+                }
+                Param::Date(d) => {
+                    values_list.write(&utils::convert_date(d.year(), d.month(), d.day()))?;
+                    blr_list.write(&[12])?;
+                }
+                Param::TimeStamp(dt) => {
+                    let d = dt.date();
+                    let t = dt.time();
+                    values_list.write(&utils::convert_date(d.year(), d.month(), d.day()))?;
+                    values_list.write(&utils::convert_time(
+                        t.hour() as u32,
+                        t.minute(),
+                        t.second(),
+                        t.nanosecond(),
+                    ))?;
+                    blr_list.write(&[35])?;
+                }
+                Param::Float(f) => {
+                    values_list.write(&utils::f32_to_bytes(*f))?;
+                    blr_list.write(&[10])?;
+                }
+                Param::Double(d) => {
+                    values_list.write(&utils::f64_to_bytes(*d))?;
+                    blr_list.write(&[27])?;
+                }
+                Param::Blob(b) => {
+                    let (blr, v) = utils::bytes_to_blr(b);
+                    values_list.write(&v)?;
+                    blr_list.write(&blr)?;
+                }
+                Param::TimeStampTZ(_dt_tz) => {
                     // TODO:
-                    panic!("another to parameter");
+                }
+                Param::Decimal(d) => {
+                    let s = d.to_string();
+                    let b = s.as_bytes();
+                    let (blr, v) = utils::bytes_to_blr(b);
+                    values_list.write(&v)?;
+                    blr_list.write(&blr)?;
+                }
+                Param::Boolean(b) => {
+                    if *b {
+                        values_list.write(&[1, 0, 0, 0])?;
+                    } else {
+                        values_list.write(&[0, 0, 0, 0])?;
+                    }
+                    blr_list.write(&[23])?;
                 }
             }
             blr_list.write(&[7, 0])?;
